"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Cursor = require("pg-cursor");
const pg_1 = require("pg");
const slonik_1 = require("slonik");
const util_1 = require("./util");
class Postgres {
    constructor(config) {
        const { database, host, port, user, password, migrations, logger = util_1.defaultLogger, idleInTransactionSessionTimeout, idleTimeoutMillis, maximumPoolSize, connectionTimeout, connectionRetryLimit, captureStackTrace, onReady } = config;
        util_1.initialize({
            host,
            port,
            database,
            user,
            password,
            logger,
            migrations,
            onReady
        });
        this.sql = slonik_1.createSqlTag({});
        this._logger = logger;
        this._pool = slonik_1.createPool(`postgresql://${user}:${password}@${host}:${port}/${database}`, {
            connectionRetryLimit,
            connectionTimeout,
            maximumPoolSize,
            idleInTransactionSessionTimeout,
            idleTimeout: idleTimeoutMillis,
            captureStackTrace,
            preferNativeBindings: false
        });
        process.on('exit', this._pool.end);
    }
    any(query) {
        return util_1.handleQuery(this._pool.any, query(this.sql), this._logger);
    }
    anyFirst(query) {
        return util_1.handleQuery(this._pool.anyFirst, query(this.sql), this._logger);
    }
    many(query) {
        return util_1.handleQuery(this._pool.many, query(this.sql), this._logger);
    }
    manyFirst(query) {
        return util_1.handleQuery(this._pool.manyFirst, query(this.sql), this._logger);
    }
    maybeOne(query) {
        return util_1.handleQuery(this._pool.maybeOne, query(this.sql), this._logger);
    }
    transaction(queries) {
        return this._pool.transaction((conn) => __awaiter(this, void 0, void 0, function* () {
            return queries.forEach((q) => __awaiter(this, void 0, void 0, function* () { return yield util_1.handleQuery(conn.query, q(this.sql), this._logger); }));
        }));
    }
    maybeOneFirst(query) {
        return util_1.handleQuery(this._pool.maybeOneFirst, query(this.sql), this._logger);
    }
    one(query) {
        return util_1.handleQuery(this._pool.one, query(this.sql), this._logger);
    }
    oneFirst(query) {
        return util_1.handleQuery(this._pool.oneFirst, query(this.sql), this._logger);
    }
    query(query) {
        return util_1.handleQuery(this._pool.query, query(this.sql), this._logger);
    }
    cursor(rows, query) {
        const { sql, values } = query(this.sql);
        return util_1.readCursor(rows, this._pool.query(new Cursor(sql, values, { rowMode: 'array', types: pg_1.types })));
    }
}
exports.Postgres = Postgres;
//# sourceMappingURL=postgres.js.map